#!/bin/bash -e
#
# This script takes no inputs; it merely FTPs into is.sci.gsfc.nasa.gov
# and retrieves the latest leapsec file for RT-STPS
#

# First, find out where we are
case $0 in
/*)  SHELLFILE=$0 ;;
./*)  SHELLFILE=${PWD}${0#.} ;;
../*) SHELLFILE=${PWD%/*}${0#..} ;;
*)  SHELLFILE=$(type -P "$0") ; if [ "${SHELLFILE:0:1}" != "/" ]; then SHELLFILE="${PWD}/$SHELLFILE" ; fi ;;
esac

# readlink -m makes sure SHELLDIR is a simple absolute path
SHELLDIR=`readlink -m "${SHELLFILE%/*}"`

# So RT-STPS home must be two levels above this directory
RTSTPS_HOME="$SHELLDIR/../.."

# Define variables here!
HOSTNAME="is.sci.gsfc.nasa.gov"
EXTERNAL="ancillary/temporal"
USERNAME="anonymous"
PASSWORD=""
scanlist="$RTSTPS_HOME/ftp_leapsec_ftp.txt"

# Define functions here!
scan_ftpdir()
{
echo "_________________________________________________________"
echo Scanning ftp://$HOSTNAME/$EXTERNAL
ftp -i -nv $HOSTNAME << EOF
user anonymous password
cd ancillary/temporal
ls -1 "$scanlist"
EOF
echo Done
echo ""
}

# Script begins below:

# 1.) Scan the FTP directory and determine the latest leapsec file
scan_ftpdir
LATEST_LEAPSEC=`cat "$scanlist" | grep -o "leapsec.*.dat" | sort | tail -n 1`
echo "Latest leapsec file is $LATEST_LEAPSEC"
echo "Downloading and updating $RTSTPS_HOME/leapsec.dat ..."

# 2.) Use wget to download the latest leapsec file, and overwrite the existing leapsec file in RT-STPS
cd "$RTSTPS_HOME"
wget -q ftp://$HOSTNAME/$EXTERNAL/$LATEST_LEAPSEC

# Do not continue if there was error
if [[ $? -ne 0 ]]; then
	echo "Error occurred during leapsec retrieval: $? . Aborting"
	rm -f "$scanlist"
	exit $?
else
	echo "Done"
fi

mv -f $LATEST_LEAPSEC leapsec.dat
rm -f "$LATEST_LEAPSEC"

# Remove the temporary file created
rm -f "$scanlist"
